use crate::database::controller::connector::establish_connection;
use crate::database::controller::members::check_member_has_access;
use crate::database::model::groups::{GroupMemberCount, RawGroup};
use crate::helper::check_access::check_access;
use crate::helper::settings::Settings;
use crate::modules::member_management::model::groups::{Group, GroupData, GroupUpdateData};
use crate::schema::groups_entities::dsl::group_id;
use crate::schema::roles_permissions_context::columns::entity;
use diesel::{ExpressionMethods, JoinOnDsl, QueryDsl, RunQueryDsl};
use rocket::State;
use std::collections::HashMap;
use diesel::associations::HasTable;

/// Returns list of all groups (database::model::groups::RawGroup)
///
/// #Arguments
/// * 'settings' - Settings, as managed State
///
/// #Returns
/// * Result<Vec<RawGroup>, diesel::result::Error> - RawGroup items inside Vector or diesel database error.
pub fn get_raw_groups(settings: &State<Settings>) -> Result<Vec<RawGroup>, diesel::result::Error> {
    use crate::schema::groups::dsl::*;
    let connection = establish_connection(settings);

    let group_vec: Result<Vec<RawGroup>, diesel::result::Error> = groups.order(group_name.asc()).load(&connection);
    match group_vec {
        Ok(group_vec) => Ok(group_vec),
        Err(e) => {
            error!("Error occured while loading groups: {}", e);
            Err(e)
        }
    }
}

/// Returns member count for a group
///
/// #Arguments
/// * 'settings' - Settings, as managed State
/// * 'group_id_to_check'
///
/// #Returns
/// * Result<i64, diesel::result::Error> - member count for group or diesel database error.
pub fn get_members_in_groups_count(
    settings: &State<Settings>,
    group_id_to_check: uuid::Uuid,
) -> Result<i64, diesel::result::Error> {
    use crate::schema::groups_entities::dsl::*;
    let connection = establish_connection(settings);

    let count: Result<i64, diesel::result::Error> = groups_entities
        .filter(group_id.eq(group_id_to_check))
        .count()
        .get_result(&connection);

    match count {
        Ok(count) => Ok(count),
        Err(e) => {
            error!("Couldn't count members in group: {}", e);
            Err(e)
        }
    }
}

/// Returns all groups with member count caller is allowed to see
///
/// #Arguments
/// * 'settings' - Settings, as managed State
/// * 'caller_uuid'
///
/// #Returns
/// * Result<Vec<GroupMemberCount, diesel::result::Error> - Vector of all groups member has access to or diesel database error.
pub fn get_group_member_count(
    settings: &State<Settings>,
    caller_id: uuid::Uuid,
) -> Result<Vec<GroupMemberCount>, diesel::result::Error> {
    let groups = get_raw_groups(settings)?;
    let mut groups_member_count: Vec<GroupMemberCount> = vec![];

    for group in groups {
        groups_member_count.push(GroupMemberCount {
            group_id: group.group_id,
            name: group.name,
            description: group.description,
            members_count: get_members_in_groups_count(settings, group.group_id)?,
            permission_edit_core: check_member_has_access(
                settings,
                caller_id,
                group.group_id,
                crate::permissions::modules::member_management::groups::core::EDIT,
            ),
            permission_delete: check_member_has_access(
                settings,
                caller_id,
                group.group_id,
                crate::permissions::modules::member_management::groups::DELETE,
            ),
            permission_view_members: check_member_has_access(
                settings,
                caller_id,
                group.group_id,
                crate::permissions::modules::member_management::groups::members::VIEW,
            ),
            permission_edit_members: check_member_has_access(
                settings,
                caller_id,
                group.group_id,
                crate::permissions::modules::member_management::groups::members::EDIT,
            ),
        })
    }

    Ok(groups_member_count)
}

/// Returns HashMap of all groups (modules::member_management::model::group::Group)
/// Calls get_raw_groups and adding selected: false for every group except default group
/// Key is group_id
///
/// #Arguments
/// * 'settings' - Settings, as managed State
///
/// #Returns
/// * HashMap<Group> - Group items inside Vector.
pub fn get_groups(
    settings: &State<Settings>,
) -> Result<HashMap<uuid::Uuid, Group>, diesel::result::Error> {
    let raw_groups: Vec<RawGroup> = get_raw_groups(settings)?;
    let mut groups: HashMap<uuid::Uuid, Group> = HashMap::new();

    for raw_group in raw_groups {
        groups.insert(
            raw_group.group_id,
            Group {
                entity_id: raw_group.group_id,
                group_name: raw_group.name,
                selected: false,
                group_description: raw_group.description,
            },
        );
    }
    Ok(groups)
}

pub fn get_groups_for_member(settings: &State<Settings>, member_id2: uuid::Uuid) -> Vec<Group> {
    //TODO: return errors
    use crate::schema::groups::dsl::*;
    use crate::schema::groups_entities::dsl::entity_id as groups_entities_entity_id;
    use crate::schema::groups_entities::dsl::group_id as groups_entities_group_id;
    use crate::schema::groups_entities::dsl::groups_entities;

    let connection = establish_connection(settings);

    let data: Result<Vec<RawGroup>, diesel::result::Error> = groups
        .left_join(groups_entities.on(group_id.eq(crate::schema::groups::dsl::entity_id)))
        .filter(groups_entities_entity_id.eq(member_id2))
        .select((entity_id, group_name, group_description))
        .load(&connection);

    match data {
        Ok(data) => {
            let mut groups_result: Vec<Group> = Vec::new();
            for raw_group in data {
                groups_result.push(Group {
                    entity_id: raw_group.group_id,
                    group_name: raw_group.name,
                    selected: false,
                    group_description: raw_group.description,
                });
            }
            groups_result
        }
        Err(e) => {
            warn!("Couldn't get groups for member: {}", e);
            vec![]
        }
    }
}

pub fn insert_group(
    settings: &State<Settings>,
    group: GroupData,
) -> Result<RawGroup, diesel::result::Error> {
    use crate::schema::entities::dsl::*;
    use crate::schema::groups::dsl::*;

    let connection = establish_connection(settings);

    let group_entity_id: uuid::Uuid = match diesel::insert_into(entities)
        .default_values()
        .returning(crate::schema::entities::dsl::entity_id)
        .get_result(&connection)
    {
        Ok(id) => id,
        Err(e) => {
            error!("Couldn't create entity_id for new group: {}", e);
            return Err(e);
        }
    };

    let inserted: Result<RawGroup, diesel::result::Error> = diesel::insert_into(groups)
        .values((
            crate::schema::groups::dsl::entity_id.eq(group_entity_id),
            group_name.eq(group.group_name.unwrap()),
            group_description.eq(group.group_description),
        ))
        .get_result(&connection);

    match inserted {
        Ok(inserted) => Ok(inserted),
        Err(e) => {
            error!("Couldn't insert new group: {}", e);
            Err(e)
        }
    }
}

pub fn delete_group(
    settings: &State<Settings>,
    group_id2: uuid::Uuid,
) -> Result<(), diesel::result::Error> {
    use crate::schema::groups::dsl::*;

    let connection = establish_connection(settings);

    match diesel::delete(groups.filter(entity_id.eq(group_id2))).execute(&connection) {
        Ok(_) => Ok(()),
        Err(e) => {
            error!("Couldn't delete group: {}", e);
            Err(e)
        }
    }
}

pub fn get_group(
    settings: &State<Settings>,
    group_id2: uuid::Uuid,
) -> Result<RawGroup, diesel::result::Error> {
    use crate::schema::groups::dsl::*;

    let connection = establish_connection(settings);

    let group: Result<RawGroup, diesel::result::Error> =
        groups.filter(entity_id.eq(group_id2)).first(&connection);

    match group {
        Ok(group) => Ok(group),
        Err(e) => {
            warn!("Couldn't load group: {}", e);
            Err(e)
        }
    }
}

pub fn update_group_core_data(settings: &State<Settings>, group_id2: uuid::Uuid, changeset: GroupUpdateData) -> Result<RawGroup, diesel::result::Error>{
    use crate::schema::groups::dsl::*;

    let connection = establish_connection(settings);

    let updated: Result<RawGroup, diesel::result::Error> = diesel::update(groups).filter(entity_id.eq(group_id2)).set(&changeset).get_result(&connection);
    match updated{
        Ok(updated) => Ok(updated),
        Err(e) => {
            error!("Couldn't update group core data: {}", e);
            Err(e)
        }
    }
}
