use std::sync::RwLock;
use std::collections::VecDeque;
use chrono::NaiveDateTime;
use std::io::{Error, Write, Read};
use std::fs::File;
use crate::helper::mail_queue::worker::send_mail;

#[derive(Deserialize, Serialize)]
pub struct MailQueue{
    pub(crate) mails: RwLock<VecDeque<Mail>>
}

#[derive(Deserialize, Serialize, Clone)]
pub struct Mail{
    pub(crate) uuid: uuid::Uuid,
    pub(crate) from: String,
    pub(crate) to: Vec<String>,
    pub(crate) subject: String,
    pub(crate) cc: Vec<String>,
    pub(crate) bcc: Vec<String>,
    pub(crate) reply_to: Option<String>,
    pub(crate) body: String,
    pub(crate) deliver_until: Option<NaiveDateTime>
}

impl MailQueue{
    pub fn add_mail(&self, mail: Mail) -> Result<(), std::sync::PoisonError<std::sync::RwLockWriteGuard<'_, std::collections::VecDeque<Mail>>>>{
        self.mails.write()?.push_back(mail);
        match self.save_to_disk(){
            Ok(_) => {}
            Err(e) => error!("Couldn't save mail_queue to disk: {}", e)
        }
        Ok(())
    }
    pub fn get_next(&self) -> Result<Option<Mail>, std::sync::PoisonError<std::sync::RwLockWriteGuard<'_, std::collections::VecDeque<Mail>>>>{
        // TODO: Check if mail expired (deliver_until check)
        Ok(self.mails.write()?.pop_front())
    }
    pub fn process_next(&self) -> Result<(), std::sync::PoisonError<std::sync::RwLockWriteGuard<'_, std::collections::VecDeque<Mail>>>>{
        let mail = self.get_next()?;
        match mail{
            Some(mail) => {
                match send_mail(mail.clone()){
                    Ok(_) => (),
                    Err(_) => {
                        return self.add_mail(mail)
                    }
                }
                Ok(())
            },
            None => Ok(())
        }
    }
    pub fn load_or_create_new() -> MailQueue {
        match std::fs::File::open("mail_queue.txt"){
            Ok(mut file) => {
                let mut contents = String::new();
                match file.read_to_string(&mut contents){
                    Ok(_) => {
                        match serde_json::from_str::<MailQueue>(&contents){
                            Ok(queue) => return queue,
                            Err(e) => {
                                panic!("Couldn't parse mail queue file: {}", e);
                            }
                        }
                    }
                    Err(e) => {
                        panic!("Couldn't read mail_queue.txt! {}", e);
                    }
                }
            }
            Err(e) => {
                warn!("Couldn't read mail queue from disk. Creating new empty queue.");
                MailQueue{
                    mails: RwLock::new(VecDeque::new())
                }
            }
        }
    }
    fn save_to_disk(&self) -> std::io::Result<()> {
        let serialized = serde_json::to_string(&self)?;
        let mut file = std::fs::File::create("mail_queue.txt")?;
        file.write_all(serialized.as_bytes())
    }
}

impl Mail{
    /// Create Mail with from, to, subject, cc, bcc, reply_to, deliver_until
    pub(crate) fn new(from: String, to: Vec<String>, subject: String, cc: Vec<String>, bcc: Vec<String>, reply_to: Option<String>, body: String, deliver_until: Option<NaiveDateTime>) -> Mail {
        Mail{
            uuid: uuid::Uuid::new_v4(),
            from,
            to,
            subject,
            cc,
            bcc,
            reply_to,
            body,
            deliver_until
        }
    }
}
