use crate::database::model::users::User;
use crate::helper::session_cookies::model::{
    SessionCookie, SessionCookieError, SessionCookieStorage,
};
use crate::modules::member_management::model::member::Member;
use chrono::{DateTime, Utc};
use core::fmt;
use rand::distributions::Alphanumeric;
use rand::{thread_rng, Rng};
use std::collections::HashMap;
use std::error;
use std::sync::RwLock;

impl SessionCookieStorage {
    /// Generates random alphanumerical id
    ///
    /// retrieve_id checks if generated id already exists in SessionCookieStorage.
    /// If so, it will invoke itself again and generate new id.
    fn retrieve_id(&self) -> String {
        let id: String = thread_rng().sample_iter(&Alphanumeric).take(30).collect();
        if self.cookies.read().unwrap().contains_key(&id) {
            self::SessionCookieStorage::retrieve_id(self)
        } else {
            id
        }
    }

    /// Generates cookie and adds it to the SessionCookieStorage
    ///
    /// Generates random id via retrieve_id, creates cookie and adds it to the Storage.
    pub fn add(&self, expires: DateTime<Utc>, user: User, member: Option<Member>) -> SessionCookie {
        let item = SessionCookie {
            id: self::SessionCookieStorage::retrieve_id(self),
            expires,
            user,
            member,
        };
        trace!(
            "Inserted cookie with id {} into SessionCookieStorage",
            item.id.clone()
        );
        self.cookies
            .write()
            .unwrap()
            .insert(item.id.clone(), item.clone());
        self.cookies
            .read()
            .unwrap()
            .get(&item.id.clone())
            .unwrap()
            .clone()
    }

    /// Removes cookie and therefore ends session
    /// Returns removed session cookie, if cookie was found
    /// Returns None if cookie didn't exist.
    pub fn remove(&self, cookie_id: String) -> Option<SessionCookie> {
        self.cookies.write().unwrap().remove(&cookie_id)
    }

    //TODO: make user don't change id while updating cookie. This would cause inconsistency
    /// Updates cookie with specified id in Storage or adds it if it doesn't exist
    pub fn update(&self, cookie: SessionCookie) -> SessionCookie {
        trace!(
            "Update cookie with id {} into SessionCookieStorage",
            cookie.id.clone()
        );
        self.cookies
            .write()
            .unwrap()
            .insert(cookie.id.clone(), cookie.clone());
        self.cookies
            .read()
            .unwrap()
            .get(&cookie.id.clone())
            .unwrap()
            .clone()
    }

    pub fn new() -> SessionCookieStorage {
        SessionCookieStorage {
            cookies: RwLock::new(HashMap::new()),
        }
    }

    /// Returns a reference to saved SessionCookie or SessionCookieError
    pub fn get(&self, id: String) -> Result<SessionCookie, SessionCookieError> {
        match self.cookies.read().unwrap().get(&id) {
            Some(cookie) => {
                if Utc::now().le(&cookie.expires) {
                    //Are we before cookie expire datetime?
                    Ok(cookie.clone())
                } else {
                    Err(SessionCookieError::Expired)
                }
            }
            None => Err(SessionCookieError::NotFound),
        }
    }
}

impl fmt::Display for SessionCookieError {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        match *self {
            SessionCookieError::Expired => write!(f, "Session expired"),
            SessionCookieError::NotFound => write!(f, "Couldn't find session with specified id"),
            SessionCookieError::Unavailable => write!(f, "SessionCookieStorage is unavailable"),
        }
    }
}

impl error::Error for SessionCookieError {
    fn source(&self) -> Option<&(dyn error::Error + 'static)> {
        None
    }
}

#[cfg(test)]
mod tests {
    use super::*;
    use chrono::NaiveDate;
    use uuid::Uuid;

    #[test]
    pub fn test_add() {
        let storage = SessionCookieStorage::new();

        let uuid = Uuid::parse_str("936DA01F9ABD4d9d80C702AF85C822A8").unwrap();

        let user = User {
            id: uuid,
            password: None,
            email: "test@test.de".to_string(),
        };

        let inserted_cookie = storage.add(
            DateTime::<Utc>::from_utc(NaiveDate::from_ymd(2099, 9, 9).and_hms(0, 00, 00), Utc),
            user.clone(),
            None,
        );

        assert!(inserted_cookie.expires.eq(&DateTime::<Utc>::from_utc(
            NaiveDate::from_ymd(2099, 9, 9).and_hms(0, 00, 00),
            Utc
        )));
        assert_eq!(inserted_cookie.user.id, user.id)
    }

    #[test]
    pub fn test_update() {
        let storage = SessionCookieStorage::new();

        let uuid = Uuid::parse_str("936DA01F9ABD4d9d80C702AF85C822A8").unwrap();

        let user = User {
            id: uuid,
            password: None,
            email: "test@test.de".to_string(),
        };

        let inserted_cookie = storage.add(
            DateTime::<Utc>::from_utc(NaiveDate::from_ymd(2099, 9, 9).and_hms(0, 00, 00), Utc),
            user.clone(),
            None,
        );

        let mut changed_cookie = inserted_cookie.clone();
        changed_cookie.user.id = Uuid::parse_str("236DA01F9ABD4d9d80C702AF85C822B3").unwrap();

        let updated_cookie = storage.update(changed_cookie.clone());

        assert_eq!(changed_cookie.user.id, updated_cookie.user.id)
    }

    #[test]
    pub fn test_expired() {
        let storage = SessionCookieStorage::new();

        let uuid = Uuid::parse_str("936DA01F9ABD4d9d80C702AF85C822A8").unwrap();

        let user = User {
            id: uuid,
            password: None,
            email: "test@test.de".to_string(),
        };

        let inserted_cookie = storage.add(
            DateTime::<Utc>::from_utc(NaiveDate::from_ymd(2000, 9, 9).and_hms(0, 00, 00), Utc),
            user.clone(),
            None,
        );

        let id = inserted_cookie.clone().id;

        assert_eq!(storage.get(id).err(), Some(SessionCookieError::Expired),)
    }

    #[test]
    pub fn test_notfound() {
        let storage = SessionCookieStorage::new();

        let uuid = Uuid::parse_str("936DA01F9ABD4d9d80C702AF85C822A8").unwrap();

        let user = User {
            id: uuid,
            password: None,
            email: "test@test.de".to_string(),
        };

        let inserted_cookie = storage.add(
            DateTime::<Utc>::from_utc(NaiveDate::from_ymd(2000, 9, 9).and_hms(0, 00, 00), Utc),
            user.clone(),
            None,
        );

        assert_eq!(
            storage.get(String::from("adf")).err(),
            Some(SessionCookieError::NotFound),
        )
    }
}
