use config::{Config, ConfigError, Environment, File};
use std::env;

#[derive(Debug, Deserialize, Default)]
pub struct Database {
    pub connection_string: String,
}

#[derive(Debug, Deserialize, Default)]
pub struct Application {
    pub url: String,
    pub default_language: String,
    pub fallback_language: String,
    pub loglevel: String,
    pub session_timeout: i64,
    pub upload_path: String,
    pub max_login_attempts: i32,
    pub login_lock_duration: i32,
    pub name: String,
    pub reset_password_token_valid_duration: i64,
}

#[derive(Debug, Deserialize, Default)]
pub struct Mail{
    pub from : String,
    pub reply_to : String,
}

#[derive(Debug, Deserialize, Default)]
pub struct Settings {
    pub database: Database,
    pub application: Application,
    pub mail: Mail
}

impl Settings {
    pub fn new() -> Result<Self, ConfigError> {
        let mut s = Config::new();

        // Start off by merging in the "default" configuration file
        s.merge(File::with_name("config/default"))?;

        // Add in the current environment file
        // Default to 'development' env
        // Note that this file is _optional_
        let env = env::var("RUN_MODE").unwrap_or("development".into());
        s.merge(File::with_name(&format!("config/{}", env)).required(false))?;

        // Add in a local configuration file
        // This file shouldn't be checked in to git
        s.merge(File::with_name("config/local").required(false))?;

        // Add in settings from the environment (with a prefix of APP)
        // Eg.. `APP_DEBUG=1 ./target/app` would set the `debug` key
        s.merge(Environment::with_prefix("app"))?;

        // Now that we're done, let's access our configuration
        // println!("debug: {:?}", s.get_bool("debug"));
        // println!("database: {:?}", s.get::<String>("database.url"));

        // You can deserialize (and thus freeze) the entire configuration as
        s.try_into()
    }
}
