#![feature(proc_macro_hygiene, decl_macro)]

extern crate config;
#[macro_use]
extern crate diesel;
#[macro_use]
extern crate log;
#[macro_use]
extern crate rocket;
extern crate serde;
extern crate serde_json;
#[macro_use]
extern crate serde_derive;
extern crate argon2;
extern crate chrono;
extern crate iban;

use crate::helper::handlebars_in_list_helper::InListHelper;
use crate::helper::session_cookies::model::SessionCookieStorage;
use helper::settings::Settings;
use rocket_contrib::serve::StaticFiles;
use rocket_contrib::templates::Template;
use std::{thread, time};
use chrono::{DateTime, Utc};
use crate::helper::mail_queue::queue::{MailQueue, Mail};
use std::sync::{PoisonError, RwLockWriteGuard, Arc};
use std::collections::VecDeque;
use rocket_contrib::templates::handlebars::{Handlebars, TemplateFileError};
use crate::helper::mail_templates::MailTemplates;

pub mod database;
pub mod helper;
pub mod modules;
pub mod permissions;
pub mod schema;

fn main() {
    env_logger::init(); //Initializes logger. Set environment variable RUST_LOG=debug|info|warn|error to set loglevel

    let settings = match Settings::new() {
        Ok(settings) => settings,
        Err(e) => {
            error!("Failed to read config: {}", e);
            std::process::exit(1);
        }
    };

    // Initialize storage for session cookies
    let cookie_storage = SessionCookieStorage::new();

    // Initialize mail queue for second thread handling outgoing mails
    // We are using Arc to access mail queue in all threads
    let mail_queue = Arc::new(MailQueue::load_or_create_new());
    let c_lock = Arc::clone(&mail_queue);

    thread::spawn(move ||{
        loop {
            match c_lock.process_next(){
                Ok(_) => {}
                Err(e) => {error!("MailQueue poisoned: {}", e)}
            }
            thread::sleep(time::Duration::from_millis(500)) // Only check for new mails ever 500 ms
        }
    });

    let mut mail_templates = MailTemplates{ registry: Handlebars::new() };
    match mail_templates.registry.register_templates_directory(".hbs", "resources/mail_templates"){
        Ok(_) => {}
        Err(e) => {
            error!("Couldn't register mail templates: {}",e);
            std::process::exit(1);
        }
    }

    rocket::ignite()
        .manage(settings)
        .manage(cookie_storage)
        .manage(mail_queue)
        .manage(mail_templates)
        .register(catchers![helper::server_errors::unauthorized, helper::server_errors::forbidden, helper::server_errors::notfound, helper::server_errors::notimplemented])
        .mount(
            "/",
            routes![
                modules::dashboard::view::dashboard,
                modules::welcome::view::welcome_get::welcome_get,
                modules::welcome::view::welcome_post::welcome_post,
                modules::welcome::view::welcome_post::password_reset_post,
                modules::welcome::view::welcome_post::password_change_post,
                modules::welcome::view::welcome_post::password_change_get,
                modules::welcome::view::login_select_get::login_select_get,
                modules::welcome::view::logout::logout_get,
                modules::member_management::view::selection_get::member_management_selection_get,
                modules::member_management::view::selection_post::member_management_selection_post,
                modules::member_management::view::profile_get::member_management_core_data_get,
                modules::member_management::view::profile_post::member_management_core_data_post,
                modules::member_management::view::groups_get::member_management_groups_get,
                modules::api::member_management::view::member_qualifications::api_member_remove_qualification,
                modules::api::member_management::view::member_qualifications::api_member_get_qualifications,
                modules::api::member_management::view::member_qualifications::api_member_get_qualifications_for_category,
                modules::api::member_management::view::member_qualifications::api_member_add_qualification,
                modules::api::member_management::view::member_driving_permissions::api_member_add_driving_license,
                modules::api::member_management::view::member_driving_permissions::api_member_add_driving_permission,
                modules::api::member_management::view::member_driving_permissions::api_member_remove_driving_license,
                modules::api::member_management::view::member_communication::api_member_get_communication_types,
                modules::api::member_management::view::member_communication::api_member_add_communication_target,
                modules::api::member_management::view::member_communication::api_member_remove_communication_target,
                modules::api::members::get_member::api_members_get,
                modules::api::members::get_member::api_members_search_name,
                modules::api::members::delete_member::api_members_delete,
                modules::api::members::member_communication::api_communication_targets_update,
                modules::api::groups::create::create_group,
                modules::api::groups::delete::delete_groups,
                modules::api::groups::read::read_group,
                modules::api::groups::update::put_member_in_group,
                modules::api::groups::delete::delete_member_from_group,
                modules::api::groups::update::update_group,
                modules::api::units::read::read_unit_list,
                modules::api::units::update::put_member_in_unit,
                modules::api::units::delete::delete_member_from_unit,
                modules::api::users::create::create_user,
                modules::api::users::delete::delete_user,
                modules::api::users::update::update_user,
                modules::member_management::view::personal_profile::member_management_personal_profile_get,
                modules::member_management::view::personal_profile::member_management_personal_profile_post,
                modules::member_management::view::create_member::member_management_add_member,
                modules::member_management::view::create_member::member_management_add_member_post,
            ],
        )
        .mount("/css", StaticFiles::from("resources/css"))
        .mount("/js", StaticFiles::from("resources/js"))//TODO: Replace popper and bootstrap with local files
        .mount("/img", StaticFiles::from("resources/images"))
        .attach(Template::custom(|engine|{
            engine.handlebars.register_helper("if_in_list", Box::new(InListHelper));
        }))
        .launch();
}
