use crate::database::controller::groups::get_group;
use crate::database::controller::groups_permissions::{
    get_group_permission_for_role, get_group_role_permissions,
};
use crate::database::controller::members::check_member_has_access;
use crate::database::controller::members_groups::get_member_search_results_in_group;
use crate::database::model::groups::RawGroup;
use crate::helper::session_cookies::model::SessionCookie;
use crate::helper::settings::Settings;
use crate::helper::translate_diesel_error::translate_diesel;
use crate::modules::api::groups::create::GroupRolePermission;
use crate::modules::api::member_management::controller::parser::{parse_member_cookie, parse_uuid};
use crate::modules::api::members::get_member::MemberSearchResult;
use crate::modules::api::model::api_outcome::{ApiError, ApiErrorWrapper};
use crate::modules::member_management::model::groups::Group;
use rocket::State;
use rocket_contrib::json::Json;

#[derive(Serialize, Deserialize, Queryable, Clone)]
pub struct CallerGroupPermissions {
    pub(crate) permission_groups_core_edit: bool,
    pub(crate) permission_groups_delete: bool,
    pub(crate) permission_groups_members_view: bool,
    pub(crate) permission_groups_members_edit: bool,
    pub(crate) permission_groups_permissions_view: bool,
    pub(crate) permission_groups_permissions_edit: bool,
}

#[derive(Serialize, Deserialize, Queryable, Clone)]
pub struct DetailedGroup {
    pub(crate) group: RawGroup,
    pub(crate) members: Option<Vec<MemberSearchResult>>,
    pub(crate) group_role_permissions: Option<Vec<GroupRolePermission>>,
    pub(crate) caller_permissions: CallerGroupPermissions,
}

#[get("/api/groups/<entity_id>", format = "json")]
pub fn read_group(
    settings: State<Settings>,
    cookie: SessionCookie,
    entity_id: String,
) -> Result<Json<DetailedGroup>, Json<ApiErrorWrapper>> {
    let caller = parse_member_cookie(cookie.member)?;
    let entity_id = parse_uuid(entity_id)?;

    if !caller
        .has_permission(crate::permissions::modules::member_management::groups::VIEW.to_string())
    {
        return Err(Json(
            ApiError::new(
                403,
                "Keine Berechtigung auf Gruppen zuzugreifen!".to_string(),
            )
            .to_wrapper(),
        ));
    }

    let caller_permissions = CallerGroupPermissions {
        permission_groups_core_edit: check_member_has_access(
            &settings,
            caller.entity_id,
            entity_id,
            crate::permissions::modules::member_management::groups::core::EDIT,
        ),
        permission_groups_delete: check_member_has_access(
            &settings,
            caller.entity_id,
            entity_id,
            crate::permissions::modules::member_management::groups::DELETE,
        ),
        permission_groups_members_view: check_member_has_access(
            &settings,
            caller.entity_id,
            entity_id,
            crate::permissions::modules::member_management::groups::members::VIEW,
        ),
        permission_groups_members_edit: check_member_has_access(
            &settings,
            caller.entity_id,
            entity_id,
            crate::permissions::modules::member_management::groups::members::EDIT,
        ),
        permission_groups_permissions_view: check_member_has_access(
            &settings,
            caller.entity_id,
            entity_id,
            crate::permissions::modules::member_management::groups::permissions::VIEW,
        ),
        permission_groups_permissions_edit: check_member_has_access(
            &settings,
            caller.entity_id,
            entity_id,
            crate::permissions::modules::member_management::groups::permissions::EDIT,
        ),
    };

    let group = match get_group(&settings, entity_id) {
        Ok(group) => group,
        Err(e) => return Err(translate_diesel(e)),
    };

    let members = match get_member_search_results_in_group(&settings, entity_id, caller.entity_id) {
        Ok(members) => members,
        Err(e) => return Err(translate_diesel(e)),
    };

    let group_role_permissions = match get_group_role_permissions(&settings, group.group_id) {
        Ok(grp) => grp,
        Err(e) => return Err(translate_diesel(e)),
    };

    let members = if caller_permissions.permission_groups_members_view {
        Some(members)
    } else {
        None
    };

    let group_role_permissions = if caller_permissions.permission_groups_permissions_view {
        Some(group_role_permissions)
    } else {
        None
    };

    Ok(Json(DetailedGroup {
        group,
        members,
        group_role_permissions,
        caller_permissions,
    }))
}
