use crate::database::controller::api_communication_targets::update_communication_target;
use crate::helper::session_cookies::model::SessionCookie;
use crate::helper::settings::Settings;
use crate::modules::api::member_management::controller::parser::{parse_member_cookie, parse_uuid};
use crate::modules::api::model::api_outcome::{ApiError, ApiErrorWrapper};
use crate::schema::communication_targets;
use diesel::result::Error;
use rocket::http::Cookies;
use rocket::State;
use rocket_contrib::json::Json;
use crate::helper::check_access::check_access;
use crate::database::controller::groups::get_groups_for_member;
use crate::database::controller::member_communication::get_communication_target;
use crate::helper::translate_diesel_error::translate_diesel;

#[derive(Serialize, Deserialize, Queryable, Clone, AsChangeset)]
#[table_name = "communication_targets"]
pub struct CommunicationTarget {
    pub(crate) target_id: Option<uuid::Uuid>,
    pub(crate) entity_id: Option<uuid::Uuid>,
    pub(crate) com_type: Option<uuid::Uuid>,
    pub(crate) value: Option<String>,
    pub(crate) description: Option<String>,
}

#[put(
    "/api/communication_targets/<target_id>",
    format = "json",
    data = "<communication_target>"
)]
pub fn api_communication_targets_update(
    settings: State<Settings>,
    cookie: SessionCookie,
    target_id: String,
    communication_target: Json<CommunicationTarget>,
) -> Result<(), Json<ApiErrorWrapper>> {
    let caller = parse_member_cookie(cookie.member)?;
    let target_id = parse_uuid(target_id)?;

    let old_target = match get_communication_target(&settings, target_id){
        Ok(target) => target,
        Err(e) => return Err(translate_diesel(e))
    };

    let member_groups = get_groups_for_member(&settings, old_target.entity_id);

    if old_target.entity_id != caller.entity_id{ //if Member edits own communication target, do not check permissions
     if !check_access(&settings, old_target.entity_id, member_groups, caller.entity_id, "modules.member_management.profile.communication.edit".to_string()) {
         return Err(Json(
             ApiError::new(
                 403,
                 "Keine Berechtigung, Kommunikation dieses Mitglieds zu bearbeiten!".to_string(),
             )
                 .to_wrapper(),
         ));
     }
    }

    let communication_target = communication_target.into_inner();

    match communication_target.value.clone() {
        Some(value) => {
            if value.is_empty() {
                return Err(Json(
                    ApiError::new(
                        400,
                        "Es wurde kein Wert für das Kommunikationsziel angegeben!".to_string(),
                    )
                    .to_wrapper(),
                ));
            }
        }
        None => {}
    }

    match update_communication_target(&settings, target_id, communication_target) {
        Ok(result) => {
            if result == 1 {
                Ok(())
            } else {
                Err(Json(
                    ApiError::new(500, "Es ist ein Datenbankfehler aufgetreten!".to_string())
                        .to_wrapper(),
                ))
            }
        }
        Err(e) => match e {
            diesel::result::Error::NotFound => Err(Json(
                ApiError::new(
                    404,
                    "Das angeforderte Kommunikationsziel konnte nicht gefunden werden!".to_string(),
                )
                .to_wrapper(),
            )),
            _ => Err(Json(
                ApiError::new(500, "Es ist ein Datenbankfehler aufgetreten!".to_string())
                    .to_wrapper(),
            )),
        },
    }
}
