use crate::database::controller::groups::get_groups;
use crate::database::controller::members::get_members;
use crate::helper::check_access::check_access_legacy;
use crate::helper::settings::Settings;
use crate::helper::sitebuilder::model::general::{Footer, Header, Script, Stylesheet};
use crate::helper::sitebuilder::model::sidebar::Sidebar;
use crate::modules::member_management::model::member::{Member, MemberWithAccess};
use crate::modules::member_management::model::member_module::MemberModuleSelection;
use rocket::http::Status;
use rocket::State;

pub fn get_selection_context(
    member: Member,
    settings: &State<Settings>,
    selected_groups: Option<Vec<String>>,
) -> Result<MemberModuleSelection, Status> {
    let header = Header {
        html_language: "de".to_string(),
        site_title: "Member List".to_string(),
        stylesheets: vec![Stylesheet {
            path: "/css/errms.css".to_string(),
        }],
    };
    let footer = Footer {
        scripts: vec![Script {
            path: "/js/member_management_delete_member.js".to_string(),
        }],
    };
    let mut sidebar = Sidebar::new(member.clone());
    sidebar.member_management.active = true;

    let mut group_list = match get_groups(settings) {
        Ok(groups) => groups,
        Err(_) => return Err(Status::InternalServerError),
    };

    match selected_groups {
        Some(groups) => {
            for group in groups {
                match group_list.get_mut(&uuid::Uuid::parse_str(&group).unwrap()) {
                    //TODO: DO NOT UNWRAP HERE
                    Some(entry) => entry.selected = true,
                    None => warn!(
                        "member management selection list: No group found for uuid {}!",
                        group
                    ),
                }
            }
        }
        None => {
            match group_list
                .get_mut(&uuid::Uuid::parse_str("00000000-0000-0000-0000-000000000000").unwrap())
            {
                Some(entry) => entry.selected = true,
                None => (),
            }
        }
    }

    let mut selected_groups: Vec<uuid::Uuid> = Vec::new();
    for group in group_list.values() {
        if group.selected == true {
            selected_groups.push(group.entity_id);
        }
    }

    let member_list = get_members(settings, selected_groups, member.clone());
    let mut member_with_access_list: Vec<MemberWithAccess> = Vec::new();

    for member_to_check in member_list {
        let read = check_access_legacy(
            settings,
            &member_to_check,
            member.entity_id,
            "modules.member_management.profile.view".to_string(),
        );

        let write = check_access_legacy(
            settings,
            &member_to_check,
            member.entity_id,
            "modules.member_management.profile.edit".to_string(),
        );

        let delete = check_access_legacy(
            settings,
            &member_to_check,
            member.entity_id,
            "modules.member_management.profile.delete".to_string(),
        );

        let mwa = MemberWithAccess {
            member: member_to_check,
            read,
            write,
            delete,
        };
        member_with_access_list.push(mwa);
    }

    Ok(MemberModuleSelection {
        header,
        footer,
        sidebar,
        member_list: member_with_access_list,
        group_list,
    })
}
